"use strict";

var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.useDropdown = void 0;
var _objectSpread2 = _interopRequireDefault(require("reshow-runtime/helpers/objectSpread2"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("reshow-runtime/helpers/objectWithoutPropertiesLoose"));
var _classLib = require("class-lib");
var _reshowHooks = require("reshow-hooks");
var _winDoc = require("win-doc");
var _react = require("react");
var _getObjectValue = _interopRequireDefault(require("get-object-value"));
var _excluded = ["simple", "alwaysOpen", "right", "upward", "item", "listStyle", "className"]; // @ts-check
/**
 * @typedef {object} DropdownHandler
 * @property {React.Ref<any>&import("react").MutableRefObject} thisEl
 * @property {React.Ref<any>&import("react").MutableRefObject} listEl
 * @property {import("react").MutableRefObject} isActive
 * @property {import("react").MouseEventHandler} dropdownClick
 * @property {Function} listClick
 * @property {Function} touchStart
 */

/**
 * @typedef {object} DropdownExpose
 * @property {Function} open
 * @property {Function} close
 * @property {Function} isOpen
 */

/**
 * @typedef {object} DropdownData
 * @property {string} className
 * @property {object} listStyle
 * @property {DropdownHandler} handler
 * @property {DropdownExpose} expose
 * @property {boolean} hideList
 * @property {object} restProps
 */

/**
 * @typedef {object} DropdownProps
 * @property {boolean} [simple]
 * @property {boolean} [alwaysOpen]
 * @property {boolean} [right]
 * @property {boolean} [upward]
 * @property {boolean} [item]
 * @property {string} [className]
 * @property {object} [listStyle]
 */

/**
 * @param {DropdownProps} props
 * @returns {DropdownData}
 */
var useDropdown = exports.useDropdown = function useDropdown(props) {
  var simple = props.simple,
    alwaysOpen = props.alwaysOpen,
    right = props.right,
    upward = props.upward,
    item = props.item,
    propsListStyle = props.listStyle,
    className = props.className,
    restProps = (0, _objectWithoutPropertiesLoose2["default"])(props, _excluded);
  var _useTimer = (0, _reshowHooks.useTimer)(),
    titleTimer = _useTimer[0];
  var _useTimer2 = (0, _reshowHooks.useTimer)(),
    menuTimer = _useTimer2[0];
  /** @type {React.MutableRefObject<EventListener | unknown>} */
  var handleClose = (0, _react.useRef)();
  /** @type {React.MutableRefObject<HTMLElement | unknown>} */
  var thisEl = (0, _react.useRef)();
  /** @type {React.MutableRefObject<HTMLElement | unknown>} */
  var listEl = (0, _react.useRef)();
  /** @type {React.MutableRefObject<Boolean | unknown>} */
  var isActive = (0, _react.useRef)();
  /** @type {React.MutableRefObject<Boolean | unknown>} */
  var isListClick = (0, _react.useRef)();
  var _useState = (0, _react.useState)({}),
    stateListStyle = _useState[0],
    setStateListStyle = _useState[1];
  var _useState2 = (0, _react.useState)(false),
    hideList = _useState2[0],
    setHideList = _useState2[1];
  (0, _react.useEffect)(function () {
    handleClose.current = function ( /** @type Event */e) {
      var target = e.target;
      if ( /** @type HTMLElement */thisEl.current.contains( /** @type HTMLElement*/target)) {
        return;
      }
      expose.close();
    };
    if (alwaysOpen) {
      expose.open();
    }
    return function () {
      if (!simple) {
        (0, _winDoc.doc)().removeEventListener("click", /** @type EventListener*/handleClose.current);
      }
    };
  }, []);

  /**
   * @type DropdownExpose
   */
  var expose = {
    open: function open() {
      var runTimeListStyle = {};
      if (simple) {
        runTimeListStyle.visibility = "hidden";
        titleTimer(function () {
          runTimeListStyle.visibility = "inherit";
          setStateListStyle((0, _objectSpread2["default"])({}, runTimeListStyle));
        }, 300);
      }
      (0, _winDoc.doc)().addEventListener("click", /** @type EventListener*/handleClose.current);
      runTimeListStyle.display = "block";
      isActive.current = true;
      setStateListStyle(runTimeListStyle);
    },
    /**
     * @param {boolean} [force]
     */
    close: function close(force) {
      if (alwaysOpen && !force) {
        return;
      }
      (0, _winDoc.doc)().removeEventListener("click", /** @type EventListener*/handleClose.current);
      isActive.current = false;
      setStateListStyle({});
    },
    isOpen: function isOpen() {
      if (simple) {
        return (0, _getObjectValue["default"])(listEl.current, ["style", "display"]) !== "";
      } else {
        return isActive.current;
      }
    }
  };

  /**
   * @type DropdownHandler
   */
  var handler = {
    thisEl: thisEl,
    listEl: listEl,
    isActive: isActive,
    dropdownClick: function dropdownClick() {
      if (!isListClick.current) {
        if (!expose.isOpen()) {
          expose.open();
        } else {
          expose.close();
        }
      } else {
        isListClick.current = false;
      }
    },
    listClick: function listClick() {
      if (simple) {
        if (!expose.isOpen()) {
          setTimeout(function () {
            setHideList(true);
            menuTimer(function () {
              return setHideList(false);
            }, 500);
          });
        }
      } else {
        expose.close();
      }
      isListClick.current = true;
    },
    /**
     * @param {Event} e
     */
    touchStart: function touchStart(e) {
      if (!expose.isOpen()) {
        e.preventDefault();
        e.stopPropagation();
      }
    }
  };
  var thisClass = (0, _classLib.mixClass)(className, {
    active: isActive.current,
    simple: simple,
    right: right,
    upward: upward,
    item: item
  });
  return {
    restProps: restProps,
    className: thisClass,
    listStyle: (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, propsListStyle), stateListStyle),
    handler: handler,
    expose: expose,
    hideList: hideList
  };
};