"use strict";

var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports.useSelect = void 0;
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("reshow-runtime/helpers/objectWithoutPropertiesLoose"));
var _react = require("react");
var _callFunc = _interopRequireDefault(require("call-func"));
var _defaultLocator = _interopRequireDefault(require("../defaultLocator.js"));
var _excluded = ["value", "defaultValue", "labelLocator", "valueLocator", "onChange", "onBeforeChange"]; // @ts-check
/**
 * @typedef {object} SelectBaseEvent
 * @property {string} [value]
 * @property {Object} [selected]
 * @property {Object} [item]
 */

/**
 * @typedef {SelectBaseEvent | SelectBaseEvent & Event} SelectEvent
 */

/**
 * @typedef {object} SelectHandler
 * @property {Function} select
 */

/**
 * @typedef {object} SelectExpose
 * @property {Function} getValue
 * @property {Function} select
 */

/**
 * @typedef {object} SelectData
 * @property {SelectHandler} handler
 * @property {SelectExpose} expose
 * @property {Function} labelLocator
 * @property {Function} valueLocator
 * @property {object} [restProps]
 */

/**
 * @typedef {object} SelectProps
 * @property {any} value
 * @property {any} defaultValue
 * @property {Function} labelLocator
 * @property {Function} valueLocator
 * @property {Function} onChange
 * @property {Function} onBeforeChange
 */

/**
 * @param {SelectProps} props
 * @returns {SelectData};
 */
var useSelect = exports.useSelect = function useSelect(props) {
  var propsValue = props.value,
    defaultValue = props.defaultValue,
    _props$labelLocator = props.labelLocator,
    labelLocator = _props$labelLocator === void 0 ? _defaultLocator["default"].label : _props$labelLocator,
    _props$valueLocator = props.valueLocator,
    valueLocator = _props$valueLocator === void 0 ? _defaultLocator["default"].value : _props$valueLocator,
    onChange = props.onChange,
    onBeforeChange = props.onBeforeChange,
    restProps = (0, _objectWithoutPropertiesLoose2["default"])(props, _excluded);
  var thisValue = propsValue !== null && propsValue !== void 0 ? propsValue : defaultValue;
  /** @type {React.MutableRefObject<SelectEvent | unknown>} */
  var lastEvent = (0, _react.useRef)();
  var lastValue = (0, _react.useRef)();
  var _useState = (0, _react.useState)(),
    value = _useState[0],
    setValue = _useState[1];
  (0, _react.useEffect)(function () {
    setValue(thisValue);
  }, [thisValue]);
  (0, _react.useEffect)(function () {
    lastValue.current = value;
    if (lastEvent.current != null) {
      (0, _callFunc["default"])(onChange, [lastEvent.current]);
    }
  }, [value]);

  /**
   * @type SelectHandler
   */
  var handler = {
    select: function select( /** @type any*/item) {
      return function ( /** @type SelectEvent*/e) {
        if (e === void 0) {
          e = {};
        }
        var value = valueLocator(item);
        var selected = labelLocator(item);
        e.value = value;
        e.selected = selected;
        e.item = item;
        lastEvent.current = e;
        var isContinue = (0, _callFunc["default"])(onBeforeChange, [e]);
        if (false !== isContinue) {
          setValue(value);
        }
      };
    }
  };

  /**
   * @type SelectExpose
   */
  var expose = {
    getValue: function getValue() {
      return lastValue.current;
    },
    select: handler.select
  };
  return {
    handler: handler,
    expose: expose,
    labelLocator: labelLocator,
    valueLocator: valueLocator,
    restProps: restProps
  };
};