"use strict";

var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
var _interopRequireWildcard = require("reshow-runtime/helpers/interopRequireWildcard");
exports.__esModule = true;
exports.getHorizontalToVerticalMenu = exports["default"] = void 0;
var _objectSpread2 = _interopRequireDefault(require("reshow-runtime/helpers/objectSpread2"));
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("reshow-runtime/helpers/objectWithoutPropertiesLoose"));
var React = _interopRequireWildcard(require("react"));
var _reactAtomicMolecule = require("react-atomic-molecule");
var _HamburgerToX = _interopRequireDefault(require("ricon/HamburgerToX"));
var _getObjectValue = _interopRequireDefault(require("get-object-value"));
var _jsxRuntime = require("react/jsx-runtime");
var _excluded = ["className"],
  _excluded2 = ["brand", "nav", "height", "component", "style", "children"]; // @ts-check
var Children = React.Children,
  useState = React.useState;

/**
 * @typedef {import('reshow-build').Component} Component
 */

/**
 * @typedef {object} HorizontalToVerticalMenuProps
 * @property {string=} className
 * @property {Component=} brand
 * @property {Component=} nav
 * @property {number=} height
 * @property {Component=} component
 * @property {React.CSSProperties=} style
 * @property {React.Children=} children
 */

/**
 * @param {{[key: string]:React.CSSProperties}} Styles
 * @param {boolean} [merge]
 */
var getHorizontalToVerticalMenu = exports.getHorizontalToVerticalMenu = function getHorizontalToVerticalMenu(Styles, merge) {
  var InjectStyles = {
    headerActive: [{
      maxHeight: "1000px !important"
    }, ".page-header.active"],
    navActive: [{
      display: "block !important"
    }, ".page-header.active .nav"],
    lgHeaderNav: [{
      display: ["inline-flex !important", "-webkit-inline-box !important", "-ms-inline-flexbox !important"]
    }, [_reactAtomicMolecule.min.lg, ".page-header .nav, .page-header.active .nav"]],
    lgHamburgerIcon: [{
      display: "none !important"
    }, [_reactAtomicMolecule.min.lg, ".page-header .hamburger-icon"]]
  };
  if (merge) {
    (0, _reactAtomicMolecule.mergeStyleConfig)(Styles, defaultStyles, InjectStyles);
  }

  /**
   * @param {HorizontalToVerticalMenuProps} props
   * @returns {{ restProps: HorizontalToVerticalMenuProps, handler: {[key: string]: Function}, className: string, active: boolean }}
   */
  var useHorizontalToVerticalMenu = function useHorizontalToVerticalMenu(props) {
    var className = props.className,
      restProps = (0, _objectWithoutPropertiesLoose2["default"])(props, _excluded);
    var _useState = useState(false),
      active = _useState[0],
      setActive = _useState[1];
    injects = (0, _reactAtomicMolecule.useLazyInject)(InjectStyles, injects);
    var classes = (0, _reactAtomicMolecule.mixClass)(className, {
      active: active
    });
    var handler = {
      on: function on() {
        setActive(function (prevActive) {
          return !prevActive;
        });
      }
    };
    return {
      restProps: restProps,
      handler: handler,
      className: classes,
      active: active
    };
  };

  /**
   * @param {HorizontalToVerticalMenuProps} props
   */
  var HorizontalToVerticalMenu = function HorizontalToVerticalMenu(props) {
    var _useHorizontalToVerti = useHorizontalToVerticalMenu(props),
      restProps = _useHorizontalToVerti.restProps,
      handler = _useHorizontalToVerti.handler,
      className = _useHorizontalToVerti.className,
      active = _useHorizontalToVerti.active;
    var _restProps$brand = restProps.brand,
      brand = _restProps$brand === void 0 ? null : _restProps$brand,
      _restProps$nav = restProps.nav,
      nav = _restProps$nav === void 0 ? null : _restProps$nav,
      _restProps$height = restProps.height,
      height = _restProps$height === void 0 ? 60 : _restProps$height,
      _restProps$component = restProps.component,
      component = _restProps$component === void 0 ? _reactAtomicMolecule.SemanticUI : _restProps$component,
      style = restProps.style,
      children = restProps.children,
      uiRestProps = (0, _objectWithoutPropertiesLoose2["default"])(restProps, _excluded2);
    var thisChildren = [];
    if (brand) {
      thisChildren.push((0, _reactAtomicMolecule.build)(brand)({
        style: (0, _objectSpread2["default"])((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, Styles.brand), (0, _getObjectValue["default"])(brand, ["props", "style"])), {}, {
          height: height
        }),
        className: (0, _reactAtomicMolecule.mixClass)((0, _getObjectValue["default"])(brand, ["props", "className"]), "brand"),
        key: "brand"
      }));
    }
    if (nav) {
      thisChildren.push((0, _reactAtomicMolecule.build)(nav)({
        style: (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, Styles.nav), (0, _getObjectValue["default"])(nav, ["props", "style"])),
        className: (0, _reactAtomicMolecule.mixClass)("nav", (0, _getObjectValue["default"])(nav, ["props", "className"])),
        key: "nav",
        handleOn: handler.on
      }));
    }
    Children.forEach(children, function (child, key) {
      thisChildren.push((0, _reactAtomicMolecule.build)( /**@type Component*/child)({
        key: key,
        handleOn: handler.on
      }));
    });
    if (nav) {
      thisChildren.push( /*#__PURE__*/(0, _jsxRuntime.jsx)(_reactAtomicMolecule.Icon, {
        className: "hamburger-icon",
        style: Styles.hamburgerIcon,
        onClick: handler.on,
        children: /*#__PURE__*/(0, _jsxRuntime.jsx)(_HamburgerToX["default"], {
          on: active
        })
      }, "hamburger-icon"));
    }
    return (0, _reactAtomicMolecule.build)(component)((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, uiRestProps), {}, {
      styles: (0, _reactAtomicMolecule.reactStyle)((0, _objectSpread2["default"])((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, Styles.container), style), {}, {
        maxHeight: height,
        transition: [["padding 300ms linear", "max-height 300ms ease-in-out"].join(", ")]
      }), undefined, false),
      className: className
    }), thisChildren);
  };
  return HorizontalToVerticalMenu;
};
var injects;
/**
 * @type {{[key: string]: React.CSSProperties}}
 */
var defaultStyles = {
  hamburgerIcon: {
    position: "absolute",
    top: 10,
    right: 30,
    width: 35,
    height: 35,
    fill: "#fff",
    cursor: "pointer"
  },
  container: {
    position: "relative",
    maxHeight: 60,
    overflow: "hidden",
    boxSizing: "border-box"
  },
  nav: {
    display: "none"
  }
};
var _default = exports["default"] = getHorizontalToVerticalMenu(defaultStyles);