"use strict";

var _interopRequireDefault = require("reshow-runtime/helpers/interopRequireDefault");
exports.__esModule = true;
exports["default"] = void 0;
var _objectWithoutPropertiesLoose2 = _interopRequireDefault(require("reshow-runtime/helpers/objectWithoutPropertiesLoose"));
var _objectSpread2 = _interopRequireDefault(require("reshow-runtime/helpers/objectSpread2"));
var _react = require("react");
var _reactAtomicMolecule = require("react-atomic-molecule");
var _getObjectValue = _interopRequireDefault(require("get-object-value"));
var _excluded = ["className", "id", "menu", "right", "footer", "style", "body"]; //@ts-check
var injects;

/**
 * @param {object} params
 * @param {object} Styles
 * @param {boolean} merge
 * @returns {DocTemplate}
 */
var getDocTemplate = function getDocTemplate(params, Styles, merge) {
  if (Styles === void 0) {
    Styles = {};
  }
  if (merge === void 0) {
    merge = true;
  }
  /* Defined containerClass */
  var sideWidth = (0, _getObjectValue["default"])(params, ["sideWidth"], 300);
  var miniSidebarWidth = (0, _getObjectValue["default"])(params, ["miniSidebarWidth"], 42);
  var rightWidth = (0, _getObjectValue["default"])(params, ["rightWidth"], 0); // use in three column layout, (Right col)
  var containerClass = "width-" + sideWidth + "-" + miniSidebarWidth + "-" + rightWidth;
  var active = (0, _getObjectValue["default"])(params, ["active"]);
  var docId = (0, _getObjectValue["default"])(params, ["docId"], "doc");
  var menuId = (0, _getObjectValue["default"])(params, ["menuId"], "doc-menu");
  var miniSidebar = (0, _getObjectValue["default"])(params, ["miniSidebar"]);
  var fixedMini = (0, _getObjectValue["default"])(params, ["fixedMini"]);
  var getMiniClass =
  /**
   * @param {string} tpl
   * @returns {string}
   */
  function getMiniClass(tpl, alwaysOn) {
    if (alwaysOn === void 0) {
      alwaysOn = fixedMini;
    }
    var c = [tpl.replace("[active]", "inactive")];
    if (alwaysOn) {
      c.push(tpl.replace("[active]", "active"));
    }
    return c.join(", ");
  };
  var initSideWidth = fixedMini ? miniSidebarWidth : active ? sideWidth : 0;
  if (initSideWidth && !isNaN(initSideWidth)) {
    initSideWidth += "px";
  }
  if (sideWidth && !isNaN(sideWidth)) {
    sideWidth += "px";
  }
  var InjectStyles = {
    /*default*/
    defaultOnIcon: [{
      display: "none"
    }, ".ui.rail>.hamburger-icon.default-on"],
    defaultOnIconSvg: [{
      display: "block"
    }, ".ui.rail>.hamburger-icon svg"],
    rail: [{
      width: 0,
      overflow: "hidden"
    }, ".ui.rail.left, #" + menuId],
    /**
     * RWD
     */
    mdContainer: [{
      padding: "0 " + rightWidth + "px 0 " + initSideWidth + " !important"
    }, [_reactAtomicMolecule.min.md, "#" + docId]],
    mdMenu: [{
      width: sideWidth + " !important"
    }, [_reactAtomicMolecule.min.md, [".ui.rail.left", "#" + menuId, "#" + menuId + " .ui.vertical.menu"].join(",")]],
    mdIconDefaultOff: [{
      display: "none !important"
    }, [_reactAtomicMolecule.min.md, ".ui.rail>.hamburger-icon.default-off"]],
    mdIconDefaultOn: [{
      left: sideWidth + " !important",
      display: "block !important"
    }, [_reactAtomicMolecule.min.md, ".ui.rail>.hamburger-icon.default-on"]],
    mdInactiveContainer: [{
      padding: "0 !important"
    }, [_reactAtomicMolecule.min.md, getMiniClass(".side-menu-[active] #" + docId)]],
    mdInactiveMenu: [{
      maxWidth: 0,
      overflowY: "hidden !important"
    }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId)]],
    mdInactiveIconDefaultOn: [{
      left: 0 + " !important"
    }, [_reactAtomicMolecule.min.md, getMiniClass(".ui.rail.[active]>.hamburger-icon.default-on")]],
    /* Active */
    activeContainer: [{
      padding: "0 " + rightWidth + "px 0 " + sideWidth + " !important",
      overflow: "hidden"
    }, ".side-menu-active #" + docId],
    activeMenu: [{
      width: sideWidth + " !important"
    }, [".active.ui > #" + menuId, ".active.ui.rail.left", ".active.ui.rail.left .ui.vertical.menu"].join(",")],
    activeIcon: [{
      left: sideWidth + " !important"
    }, ".active > #" + menuId + " ~ .hamburger-icon"]
  };
  if (miniSidebar) {
    InjectStyles = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, InjectStyles), {}, {
      /* RWD Inactive */
      mdInactiveContainer: [{
        padding: "0 0 0 " + miniSidebarWidth + "px !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".side-menu-[active] #" + docId)]],
      mdRailInactive: [{
        maxWidth: miniSidebarWidth
      }, [_reactAtomicMolecule.min.md, getMiniClass("#" + docId + " > .ui.rail.[active]")]],
      mdInactiveMenu: [{
        maxWidth: miniSidebarWidth,
        overflowY: "hidden !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId)]],
      mdInactiveMenuLink: [{
        padding: "5px 0 !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId + " .item > a")]],
      mdInactiveSubMenu: [{
        display: "none !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId + " .item > .menu")]],
      mdInactiveHover: [{
        overflow: "visible !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId + ":hover")]],
      mdInactiveItemHover: [{
        display: "block !important",
        position: "absolute",
        minWidth: 224,
        maxHeight: "50vh",
        overflow: "auto",
        top: 0,
        left: miniSidebarWidth,
        background: "#fff",
        borderRadius: 5,
        border: "1px solid #eee"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".[active] #" + menuId + " .item:hover > .menu")]],
      mdInactiveIconDefaultOn: [{
        left: miniSidebarWidth + 1 + " px !important"
      }, [_reactAtomicMolecule.min.md, getMiniClass(".ui.rail.[active]>.hamburger-icon.default-on")]]
    });
    if (fixedMini) {
      InjectStyles = (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, InjectStyles), {}, {
        mdVerticalMenu: [{
          overflowY: "scroll",
          overflowX: "hidden",
          maxHeight: "100vh"
        }, "#" + menuId + " .ui.vertical.menu"],
        mdInactiveHover: [{
          overflow: "hidden !important"
        }, getMiniClass(".[active] #" + menuId + ":hover")],
        /**
         * @type any
         */
        mdInactiveMenu: [{
          maxWidth: miniSidebarWidth,
          overflowY: "hidden !important"
        }, getMiniClass(".[active] #" + menuId)],
        activeIcon: [{
          left: miniSidebarWidth + "px !important"
        }, ".active > #" + menuId + " ~ .hamburger-icon"]
      });
    }
  }
  if (merge) {
    (0, _reactAtomicMolecule.mergeStyleConfig)(Styles, defaultStyles, InjectStyles);
  }

  /**
   * @returns {React.ReactComponentElement}
   */
  var DocTemplate = function DocTemplate(_ref) {
    var _ref$className = _ref.className,
      className = _ref$className === void 0 ? "basic" : _ref$className,
      _ref$id = _ref.id,
      id = _ref$id === void 0 ? docId : _ref$id,
      _ref$menu = _ref.menu,
      menu = _ref$menu === void 0 ? null : _ref$menu,
      _ref$right = _ref.right,
      right = _ref$right === void 0 ? null : _ref$right,
      _ref$footer = _ref.footer,
      footer = _ref$footer === void 0 ? null : _ref$footer,
      _ref$style = _ref.style,
      style = _ref$style === void 0 ? {} : _ref$style,
      body = _ref.body,
      restProps = (0, _objectWithoutPropertiesLoose2["default"])(_ref, _excluded);
    injects = (0, _reactAtomicMolecule.useLazyInject)(InjectStyles, injects);
    var thisBody = (0, _react.useMemo)(function () {
      return body ? (0, _reactAtomicMolecule.build)(body)({
        key: "body"
      }) : null;
    }, [body]);
    var thisFooter = (0, _react.useMemo)(function () {
      return footer ? (0, _reactAtomicMolecule.build)(footer)({
        key: "footer"
      }) : null;
    }, [footer]);
    var thisMenu = (0, _react.useMemo)(function () {
      return menu ? (0, _reactAtomicMolecule.build)(menu)({
        key: "menu"
      }) : null;
    }, [menu]);
    var thisRight = (0, _react.useMemo)(function () {
      return right ? (0, _reactAtomicMolecule.build)(right)({
        key: "right"
      }) : null;
    }, [right]);
    var thisStyle = (0, _react.useMemo)(function () {
      return (0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, Styles.container), style);
    }, [style]);
    return (0, _reactAtomicMolecule.build)(_reactAtomicMolecule.SemanticUI)((0, _objectSpread2["default"])((0, _objectSpread2["default"])({}, restProps), {}, {
      className: (0, _reactAtomicMolecule.mixClass)(className, containerClass),
      style: thisStyle,
      id: id
    }), [(0, _reactAtomicMolecule.build)(_reactAtomicMolecule.SemanticUI)({
      key: "doc-body",
      className: "doc-body",
      style: Styles.docBody
    }, [thisBody, thisFooter]), thisMenu, thisRight]);
  };
  return DocTemplate;
};
var defaultStyles = {
  container: {
    margin: 0
  },
  docBody: {
    minWidth: 350
  }
};
var _default = exports["default"] = getDocTemplate;
module.exports = exports.default;