import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutProperties from "reshow-runtime/es/helpers/objectWithoutProperties";
var _excluded = ["simple", "alwaysOpen", "right", "upward", "item", "listStyle", "className"];
// @ts-check

import { mixClass } from "class-lib";
import { useTimer } from "reshow-hooks";
import { doc } from "win-doc";
import { useState, useEffect, useRef } from "react";
import get from "get-object-value";

/**
 * @typedef {object} DropdownHandler
 * @property {React.Ref<any>&import("react").MutableRefObject} thisEl
 * @property {React.Ref<any>&import("react").MutableRefObject} listEl
 * @property {import("react").MutableRefObject} isActive
 * @property {import("react").MouseEventHandler} dropdownClick
 * @property {Function} listClick
 * @property {Function} touchStart
 */

/**
 * @typedef {object} DropdownExpose
 * @property {Function} open
 * @property {Function} close
 * @property {Function} isOpen
 */

/**
 * @typedef {object} DropdownData
 * @property {string} className
 * @property {object} listStyle
 * @property {DropdownHandler} handler
 * @property {DropdownExpose} expose
 * @property {boolean} hideList
 * @property {object} restProps
 */

/**
 * @typedef {object} DropdownProps
 * @property {boolean} [simple]
 * @property {boolean} [alwaysOpen]
 * @property {boolean} [right]
 * @property {boolean} [upward]
 * @property {boolean} [item]
 * @property {string} [className]
 * @property {object} [listStyle]
 */

/**
 * @param {DropdownProps} props
 * @returns {DropdownData}
 */
export var useDropdown = function useDropdown(props) {
  var {
      simple,
      alwaysOpen,
      right,
      upward,
      item,
      listStyle: propsListStyle,
      className
    } = props,
    restProps = _objectWithoutProperties(props, _excluded);
  var [titleTimer] = useTimer();
  var [menuTimer] = useTimer();
  /** @type {React.MutableRefObject<EventListener | unknown>} */
  var handleClose = useRef();
  /** @type {React.MutableRefObject<HTMLElement | unknown>} */
  var thisEl = useRef();
  /** @type {React.MutableRefObject<HTMLElement | unknown>} */
  var listEl = useRef();
  /** @type {React.MutableRefObject<Boolean | unknown>} */
  var isActive = useRef();
  /** @type {React.MutableRefObject<Boolean | unknown>} */
  var isListClick = useRef();
  var [stateListStyle, setStateListStyle] = useState({});
  var [hideList, setHideList] = useState(false);
  useEffect(function () {
    handleClose.current = function ( /** @type Event */e) {
      var target = e.target;
      if ( /** @type HTMLElement */thisEl.current.contains( /** @type HTMLElement*/target)) {
        return;
      }
      expose.close();
    };
    if (alwaysOpen) {
      expose.open();
    }
    return function () {
      if (!simple) {
        doc().removeEventListener("click", /** @type EventListener*/handleClose.current);
      }
    };
  }, []);

  /**
   * @type DropdownExpose
   */
  var expose = {
    open: function open() {
      var runTimeListStyle = {};
      if (simple) {
        runTimeListStyle.visibility = "hidden";
        titleTimer(function () {
          runTimeListStyle.visibility = "inherit";
          setStateListStyle(_objectSpread({}, runTimeListStyle));
        }, 300);
      }
      doc().addEventListener("click", /** @type EventListener*/handleClose.current);
      runTimeListStyle.display = "block";
      isActive.current = true;
      setStateListStyle(runTimeListStyle);
    },
    /**
     * @param {boolean} [force]
     */
    close: function close(force) {
      if (alwaysOpen && !force) {
        return;
      }
      doc().removeEventListener("click", /** @type EventListener*/handleClose.current);
      isActive.current = false;
      setStateListStyle({});
    },
    isOpen: function isOpen() {
      if (simple) {
        return get(listEl.current, ["style", "display"]) !== "";
      } else {
        return isActive.current;
      }
    }
  };

  /**
   * @type DropdownHandler
   */
  var handler = {
    thisEl,
    listEl,
    isActive,
    dropdownClick: function dropdownClick() {
      if (!isListClick.current) {
        if (!expose.isOpen()) {
          expose.open();
        } else {
          expose.close();
        }
      } else {
        isListClick.current = false;
      }
    },
    listClick: function listClick() {
      if (simple) {
        if (!expose.isOpen()) {
          setTimeout(function () {
            setHideList(true);
            menuTimer(function () {
              return setHideList(false);
            }, 500);
          });
        }
      } else {
        expose.close();
      }
      isListClick.current = true;
    },
    /**
     * @param {Event} e
     */
    touchStart: function touchStart(e) {
      if (!expose.isOpen()) {
        e.preventDefault();
        e.stopPropagation();
      }
    }
  };
  var thisClass = mixClass(className, {
    active: isActive.current,
    simple,
    right,
    upward,
    item
  });
  return {
    restProps,
    className: thisClass,
    listStyle: _objectSpread(_objectSpread({}, propsListStyle), stateListStyle),
    handler,
    expose,
    hideList
  };
};