import _objectWithoutProperties from "reshow-runtime/es/helpers/objectWithoutProperties";
var _excluded = ["value", "defaultValue", "labelLocator", "valueLocator", "onChange", "onBeforeChange"];
// @ts-check

import { useState, useEffect, useRef } from "react";
import callfunc from "call-func";
import defaultLocator from "../defaultLocator.mjs";

/**
 * @typedef {object} SelectBaseEvent
 * @property {string} [value]
 * @property {Object} [selected]
 * @property {Object} [item]
 */

/**
 * @typedef {SelectBaseEvent | SelectBaseEvent & Event} SelectEvent
 */

/**
 * @typedef {object} SelectHandler
 * @property {Function} select
 */

/**
 * @typedef {object} SelectExpose
 * @property {Function} getValue
 * @property {Function} select
 */

/**
 * @typedef {object} SelectData
 * @property {SelectHandler} handler
 * @property {SelectExpose} expose
 * @property {Function} labelLocator
 * @property {Function} valueLocator
 * @property {object} [restProps]
 */

/**
 * @typedef {object} SelectProps
 * @property {any} value
 * @property {any} defaultValue
 * @property {Function} labelLocator
 * @property {Function} valueLocator
 * @property {Function} onChange
 * @property {Function} onBeforeChange
 */

/**
 * @param {SelectProps} props
 * @returns {SelectData};
 */
export var useSelect = function useSelect(props) {
  var {
      value: propsValue,
      defaultValue,
      labelLocator = defaultLocator.label,
      valueLocator = defaultLocator.value,
      onChange,
      onBeforeChange
    } = props,
    restProps = _objectWithoutProperties(props, _excluded);
  var thisValue = propsValue !== null && propsValue !== void 0 ? propsValue : defaultValue;
  /** @type {React.MutableRefObject<SelectEvent | unknown>} */
  var lastEvent = useRef();
  var lastValue = useRef();
  var [value, setValue] = useState();
  useEffect(function () {
    setValue(thisValue);
  }, [thisValue]);
  useEffect(function () {
    lastValue.current = value;
    if (lastEvent.current != null) {
      callfunc(onChange, [lastEvent.current]);
    }
  }, [value]);

  /**
   * @type SelectHandler
   */
  var handler = {
    select: function select( /** @type any*/item) {
      return function ( /** @type SelectEvent*/e) {
        if (e === void 0) {
          e = {};
        }
        var value = valueLocator(item);
        var selected = labelLocator(item);
        e.value = value;
        e.selected = selected;
        e.item = item;
        lastEvent.current = e;
        var isContinue = callfunc(onBeforeChange, [e]);
        if (false !== isContinue) {
          setValue(value);
        }
      };
    }
  };

  /**
   * @type SelectExpose
   */
  var expose = {
    getValue: function getValue() {
      return lastValue.current;
    },
    select: handler.select
  };
  return {
    handler,
    expose,
    labelLocator,
    valueLocator,
    restProps
  };
};