import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import _objectWithoutProperties from "reshow-runtime/es/helpers/objectWithoutProperties";
var _excluded = ["className"],
  _excluded2 = ["brand", "nav", "height", "component", "style", "children"];
// @ts-check
import * as React from "react";
var {
  Children,
  useState
} = React;
import { build, min, useLazyInject, mixClass, mergeStyleConfig, reactStyle, Icon, SemanticUI } from "react-atomic-molecule";
import HamburgerIcon from "ricon/HamburgerToX";
import get from "get-object-value";

/**
 * @typedef {import('reshow-build').Component} Component
 */

/**
 * @typedef {object} HorizontalToVerticalMenuProps
 * @property {string=} className
 * @property {Component=} brand
 * @property {Component=} nav
 * @property {number=} height
 * @property {Component=} component
 * @property {React.CSSProperties=} style
 * @property {React.Children=} children
 */

/**
 * @param {{[key: string]:React.CSSProperties}} Styles
 * @param {boolean} [merge]
 */
import { jsx as _jsx } from "react/jsx-runtime";
export var getHorizontalToVerticalMenu = function getHorizontalToVerticalMenu(Styles, merge) {
  var InjectStyles = {
    headerActive: [{
      maxHeight: "1000px !important"
    }, ".page-header.active"],
    navActive: [{
      display: "block !important"
    }, ".page-header.active .nav"],
    lgHeaderNav: [{
      display: ["inline-flex !important", "-webkit-inline-box !important", "-ms-inline-flexbox !important"]
    }, [min.lg, ".page-header .nav, .page-header.active .nav"]],
    lgHamburgerIcon: [{
      display: "none !important"
    }, [min.lg, ".page-header .hamburger-icon"]]
  };
  if (merge) {
    mergeStyleConfig(Styles, defaultStyles, InjectStyles);
  }

  /**
   * @param {HorizontalToVerticalMenuProps} props
   * @returns {{ restProps: HorizontalToVerticalMenuProps, handler: {[key: string]: Function}, className: string, active: boolean }}
   */
  var useHorizontalToVerticalMenu = function useHorizontalToVerticalMenu(props) {
    var {
        className
      } = props,
      restProps = _objectWithoutProperties(props, _excluded);
    var [active, setActive] = useState(false);
    injects = useLazyInject(InjectStyles, injects);
    var classes = mixClass(className, {
      active
    });
    var handler = {
      on: function on() {
        setActive(function (prevActive) {
          return !prevActive;
        });
      }
    };
    return {
      restProps,
      handler,
      className: classes,
      active
    };
  };

  /**
   * @param {HorizontalToVerticalMenuProps} props
   */
  var HorizontalToVerticalMenu = function HorizontalToVerticalMenu(props) {
    var {
      restProps,
      handler,
      className,
      active
    } = useHorizontalToVerticalMenu(props);
    var {
        brand = null,
        nav = null,
        height = 60,
        component = SemanticUI,
        style,
        children
      } = restProps,
      uiRestProps = _objectWithoutProperties(restProps, _excluded2);
    var thisChildren = [];
    if (brand) {
      thisChildren.push(build(brand)({
        style: _objectSpread(_objectSpread(_objectSpread({}, Styles.brand), get(brand, ["props", "style"])), {}, {
          height: height
        }),
        className: mixClass(get(brand, ["props", "className"]), "brand"),
        key: "brand"
      }));
    }
    if (nav) {
      thisChildren.push(build(nav)({
        style: _objectSpread(_objectSpread({}, Styles.nav), get(nav, ["props", "style"])),
        className: mixClass("nav", get(nav, ["props", "className"])),
        key: "nav",
        handleOn: handler.on
      }));
    }
    Children.forEach(children, function (child, key) {
      thisChildren.push(build( /**@type Component*/child)({
        key,
        handleOn: handler.on
      }));
    });
    if (nav) {
      thisChildren.push( /*#__PURE__*/_jsx(Icon, {
        className: "hamburger-icon",
        style: Styles.hamburgerIcon,
        onClick: handler.on,
        children: /*#__PURE__*/_jsx(HamburgerIcon, {
          on: active
        })
      }, "hamburger-icon"));
    }
    return build(component)(_objectSpread(_objectSpread({}, uiRestProps), {}, {
      styles: reactStyle(_objectSpread(_objectSpread(_objectSpread({}, Styles.container), style), {}, {
        maxHeight: height,
        transition: [["padding 300ms linear", "max-height 300ms ease-in-out"].join(", ")]
      }), undefined, false),
      className
    }), thisChildren);
  };
  return HorizontalToVerticalMenu;
};
var injects;
/**
 * @type {{[key: string]: React.CSSProperties}}
 */
var defaultStyles = {
  hamburgerIcon: {
    position: "absolute",
    top: 10,
    right: 30,
    width: 35,
    height: 35,
    fill: "#fff",
    cursor: "pointer"
  },
  container: {
    position: "relative",
    maxHeight: 60,
    overflow: "hidden",
    boxSizing: "border-box"
  },
  nav: {
    display: "none"
  }
};
export default getHorizontalToVerticalMenu(defaultStyles);