import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
import { forwardRef, useImperativeHandle, useState, useEffect, useRef, Children } from "react";
import { mixClass, build, useCSS, SemanticUI } from "react-atomic-molecule";
import callfunc from "call-func";
import { jsx as _jsx } from "react/jsx-runtime";
var handleTabPress = function handleTabPress(_ref) {
  var {
    disableSwitch,
    nodeKey,
    nodeProps,
    setLastSelected,
    thisSelected,
    onTabItemPress,
    onTabItemWillPress
  } = _ref;
  return function (e) {
    var isContinue = false !== callfunc(onTabItemWillPress, [{
      nodeKey,
      selected: thisSelected.current
    }]);
    if (!disableSwitch && isContinue) {
      if (!nodeProps.disableSwitch) {
        setLastSelected(nodeKey);
        thisSelected.current = nodeKey;
      }
    }
    e.selected = thisSelected.current;
    e.nodeKey = nodeKey;
    callfunc(onTabItemPress, [thisSelected.current, e]);
  };
};
var handleSelected = function handleSelected(_ref2) {
  var {
    children,
    nextSelect,
    setLastSelected,
    thisSelected,
    disableSwitch,
    onTabItemWillPress,
    onTabItemPress
  } = _ref2;
  var tabMenuItems = [];
  var contentView = null;
  var hasSelected = false;
  Children.map(children, function (item, itemKey) {
    var itemProps = item.props;
    // Detect selected
    var nodeKey = "name" in itemProps ? itemProps.name : itemKey;
    if (true === nextSelect) {
      nextSelect = nodeKey;
    }
    thisSelected.current = nextSelect;
    var isActived = nodeKey === nextSelect;
    if (isActived && !hasSelected) {
      hasSelected = true;
    }
    Children.map(itemProps.children, function (node, index) {
      if (index % 2 || 1 === Children.count(itemProps.children)) {
        var _node;
        var nodeProps = ((_node = node) === null || _node === void 0 ? void 0 : _node.props) || {};
        var nodeClasses = mixClass(nodeProps.className, "item", {
          active: isActived
        });
        node = build(node)({
          key: nodeKey,
          "data-selected": isActived,
          className: nodeClasses,
          style: _objectSpread(_objectSpread({}, Styles.tabItem), nodeProps.style),
          onClickCapture: handleTabPress({
            disableSwitch,
            nodeKey,
            nodeProps,
            setLastSelected,
            thisSelected,
            onTabItemWillPress,
            onTabItemPress
          })
        });
        tabMenuItems.push(node);
      } else {
        if (isActived) {
          contentView = node;
        }
      }
    });
  });
  return {
    hasSelected,
    contentView,
    tabMenuItems
  };
};
var useTabView = function useTabView(props) {
  var {
    disableSwitch = false,
    rwd = true,
    selected: propsSelected = true,
    menu = SemanticUI,
    body = SemanticUI,
    id,
    style,
    children,
    contentStyle,
    menuStyle,
    leftMenu,
    rightMenu,
    bottom,
    left,
    right,
    onTabItemWillPress,
    onTabItemPress,
    onChange
  } = props;
  useCSS(["tab"], "semantic");
  var [lastSelected, setHookLastSelected] = useState({
    current: propsSelected
  });
  var lastPropsSelected = useRef();
  var thisSelected = useRef();
  var setLastSelected = function setLastSelected(val) {
    return setHookLastSelected(function (_ref3) {
      var {
        prev,
        current
      } = _ref3;
      return {
        prev: current,
        current: val
      };
    });
  };
  useEffect(function () {
    if (propsSelected !== lastPropsSelected.current) {
      lastPropsSelected.current = propsSelected;
      setLastSelected(propsSelected);
    }
  }, [propsSelected]);
  useEffect(function () {
    if (lastSelected.prev !== lastSelected.current) {
      thisSelected.current = lastSelected.current;
      setTimeout(function () {
        callfunc(onChange, [thisSelected.current, {
          selected: thisSelected.current
        }]);
      });
    }
  }, [lastSelected]);
  var selectResult = handleSelected({
    nextSelect: lastSelected.current,
    children,
    thisSelected,
    setLastSelected,
    disableSwitch,
    onTabItemWillPress,
    onTabItemPress
  });
  if (!selectResult.hasSelected && propsSelected !== false) {
    selectResult = handleSelected({
      nextSelect: true,
      children,
      thisSelected,
      setLastSelected,
      disableSwitch,
      onTabItemWillPress,
      onTabItemPress
    });
    if (selectResult.hasSelected) {
      setTimeout(function () {
        return setLastSelected(thisSelected.current);
      });
    }
  }
  var {
    contentView,
    tabMenuItems
  } = selectResult;
  // Tab Menu
  if (leftMenu) {
    tabMenuItems.push(build(leftMenu)({
      key: "l-menu"
    }));
  }
  if (rightMenu) {
    tabMenuItems.push(build(rightMenu)({
      key: "r-menu"
    }));
  }
  var menuOpt = {
    top: !bottom && !left && !right,
    bottom,
    vertical: left || right,
    stackable: rwd
  };
  var expose = {
    getSelected: function getSelected() {
      return thisSelected.current;
    }
  };
  return {
    contentView,
    tabMenuItems,
    menuOpt,
    menuStyle,
    menu,
    body,
    contentStyle,
    expose,
    id,
    style
  };
};
var TabView = /*#__PURE__*/forwardRef(function (props, ref) {
  var {
    contentView,
    tabMenuItems,
    menuOpt,
    menuStyle,
    menu,
    body,
    contentStyle,
    expose,
    id,
    style
  } = useTabView(props);
  useImperativeHandle(ref, function () {
    return expose;
  }, []);
  var menuClasses = mixClass("attached tabular menu", menuOpt);
  var tabMenu = build(menu)({
    key: "menu",
    style: menuStyle,
    className: menuClasses
  }, tabMenuItems);
  var content;
  if (contentView) {
    // Tab Body
    var contentClasses = mixClass("attached tab segment active", {
      top: !menuOpt.top,
      bottom: menuOpt.top
    });
    content = build(body)({
      key: "content",
      style: _objectSpread(_objectSpread({}, Styles.tabBody), contentStyle),
      className: contentClasses
    }, contentView);
  }
  var childOrder = menuOpt.top ? [tabMenu, content] : [content, tabMenu];
  return /*#__PURE__*/_jsx(SemanticUI, {
    style: style,
    id: id,
    className: "tab-view",
    children: childOrder
  });
});
TabView.displayName = "TabView";
export default TabView;
var Styles = {
  tabBody: {
    boxSizing: "border-box"
  },
  tabItem: {
    boxSizing: "border-box",
    cursor: "pointer"
  }
};