import _objectWithoutProperties from "reshow-runtime/es/helpers/objectWithoutProperties";
import _objectSpread from "reshow-runtime/es/helpers/objectSpread2";
var _excluded = ["className", "id", "menu", "right", "footer", "style", "body"];
//@ts-check

import { useMemo } from "react";
import { build, min, mixClass, mergeStyleConfig, useLazyInject, SemanticUI } from "react-atomic-molecule";
import get from "get-object-value";
var injects;

/**
 * @param {object} params
 * @param {object} Styles
 * @param {boolean} merge
 * @returns {DocTemplate}
 */
var getDocTemplate = function getDocTemplate(params, Styles, merge) {
  if (Styles === void 0) {
    Styles = {};
  }
  if (merge === void 0) {
    merge = true;
  }
  /* Defined containerClass */
  var sideWidth = get(params, ["sideWidth"], 300);
  var miniSidebarWidth = get(params, ["miniSidebarWidth"], 42);
  var rightWidth = get(params, ["rightWidth"], 0); // use in three column layout, (Right col)
  var containerClass = "width-" + sideWidth + "-" + miniSidebarWidth + "-" + rightWidth;
  var active = get(params, ["active"]);
  var docId = get(params, ["docId"], "doc");
  var menuId = get(params, ["menuId"], "doc-menu");
  var miniSidebar = get(params, ["miniSidebar"]);
  var fixedMini = get(params, ["fixedMini"]);
  var getMiniClass =
  /**
   * @param {string} tpl
   * @returns {string}
   */
  function getMiniClass(tpl, alwaysOn) {
    if (alwaysOn === void 0) {
      alwaysOn = fixedMini;
    }
    var c = [tpl.replace("[active]", "inactive")];
    if (alwaysOn) {
      c.push(tpl.replace("[active]", "active"));
    }
    return c.join(", ");
  };
  var initSideWidth = fixedMini ? miniSidebarWidth : active ? sideWidth : 0;
  if (initSideWidth && !isNaN(initSideWidth)) {
    initSideWidth += "px";
  }
  if (sideWidth && !isNaN(sideWidth)) {
    sideWidth += "px";
  }
  var InjectStyles = {
    /*default*/
    defaultOnIcon: [{
      display: "none"
    }, ".ui.rail>.hamburger-icon.default-on"],
    defaultOnIconSvg: [{
      display: "block"
    }, ".ui.rail>.hamburger-icon svg"],
    rail: [{
      width: 0,
      overflow: "hidden"
    }, ".ui.rail.left, #" + menuId],
    /**
     * RWD
     */
    mdContainer: [{
      padding: "0 " + rightWidth + "px 0 " + initSideWidth + " !important"
    }, [min.md, "#" + docId]],
    mdMenu: [{
      width: sideWidth + " !important"
    }, [min.md, [".ui.rail.left", "#" + menuId, "#" + menuId + " .ui.vertical.menu"].join(",")]],
    mdIconDefaultOff: [{
      display: "none !important"
    }, [min.md, ".ui.rail>.hamburger-icon.default-off"]],
    mdIconDefaultOn: [{
      left: sideWidth + " !important",
      display: "block !important"
    }, [min.md, ".ui.rail>.hamburger-icon.default-on"]],
    mdInactiveContainer: [{
      padding: "0 !important"
    }, [min.md, getMiniClass(".side-menu-[active] #" + docId)]],
    mdInactiveMenu: [{
      maxWidth: 0,
      overflowY: "hidden !important"
    }, [min.md, getMiniClass(".[active] #" + menuId)]],
    mdInactiveIconDefaultOn: [{
      left: 0 + " !important"
    }, [min.md, getMiniClass(".ui.rail.[active]>.hamburger-icon.default-on")]],
    /* Active */
    activeContainer: [{
      padding: "0 " + rightWidth + "px 0 " + sideWidth + " !important",
      overflow: "hidden"
    }, ".side-menu-active #" + docId],
    activeMenu: [{
      width: sideWidth + " !important"
    }, [".active.ui > #" + menuId, ".active.ui.rail.left", ".active.ui.rail.left .ui.vertical.menu"].join(",")],
    activeIcon: [{
      left: sideWidth + " !important"
    }, ".active > #" + menuId + " ~ .hamburger-icon"]
  };
  if (miniSidebar) {
    InjectStyles = _objectSpread(_objectSpread({}, InjectStyles), {}, {
      /* RWD Inactive */
      mdInactiveContainer: [{
        padding: "0 0 0 " + miniSidebarWidth + "px !important"
      }, [min.md, getMiniClass(".side-menu-[active] #" + docId)]],
      mdRailInactive: [{
        maxWidth: miniSidebarWidth
      }, [min.md, getMiniClass("#" + docId + " > .ui.rail.[active]")]],
      mdInactiveMenu: [{
        maxWidth: miniSidebarWidth,
        overflowY: "hidden !important"
      }, [min.md, getMiniClass(".[active] #" + menuId)]],
      mdInactiveMenuLink: [{
        padding: "5px 0 !important"
      }, [min.md, getMiniClass(".[active] #" + menuId + " .item > a")]],
      mdInactiveSubMenu: [{
        display: "none !important"
      }, [min.md, getMiniClass(".[active] #" + menuId + " .item > .menu")]],
      mdInactiveHover: [{
        overflow: "visible !important"
      }, [min.md, getMiniClass(".[active] #" + menuId + ":hover")]],
      mdInactiveItemHover: [{
        display: "block !important",
        position: "absolute",
        minWidth: 224,
        maxHeight: "50vh",
        overflow: "auto",
        top: 0,
        left: miniSidebarWidth,
        background: "#fff",
        borderRadius: 5,
        border: "1px solid #eee"
      }, [min.md, getMiniClass(".[active] #" + menuId + " .item:hover > .menu")]],
      mdInactiveIconDefaultOn: [{
        left: miniSidebarWidth + 1 + " px !important"
      }, [min.md, getMiniClass(".ui.rail.[active]>.hamburger-icon.default-on")]]
    });
    if (fixedMini) {
      InjectStyles = _objectSpread(_objectSpread({}, InjectStyles), {}, {
        mdVerticalMenu: [{
          overflowY: "scroll",
          overflowX: "hidden",
          maxHeight: "100vh"
        }, "#" + menuId + " .ui.vertical.menu"],
        mdInactiveHover: [{
          overflow: "hidden !important"
        }, getMiniClass(".[active] #" + menuId + ":hover")],
        /**
         * @type any
         */
        mdInactiveMenu: [{
          maxWidth: miniSidebarWidth,
          overflowY: "hidden !important"
        }, getMiniClass(".[active] #" + menuId)],
        activeIcon: [{
          left: miniSidebarWidth + "px !important"
        }, ".active > #" + menuId + " ~ .hamburger-icon"]
      });
    }
  }
  if (merge) {
    mergeStyleConfig(Styles, defaultStyles, InjectStyles);
  }

  /**
   * @returns {React.ReactComponentElement}
   */
  var DocTemplate = function DocTemplate(_ref) {
    var {
        className = "basic",
        id = docId,
        menu = null,
        right = null,
        footer = null,
        style = {},
        body
      } = _ref,
      restProps = _objectWithoutProperties(_ref, _excluded);
    injects = useLazyInject(InjectStyles, injects);
    var thisBody = useMemo(function () {
      return body ? build(body)({
        key: "body"
      }) : null;
    }, [body]);
    var thisFooter = useMemo(function () {
      return footer ? build(footer)({
        key: "footer"
      }) : null;
    }, [footer]);
    var thisMenu = useMemo(function () {
      return menu ? build(menu)({
        key: "menu"
      }) : null;
    }, [menu]);
    var thisRight = useMemo(function () {
      return right ? build(right)({
        key: "right"
      }) : null;
    }, [right]);
    var thisStyle = useMemo(function () {
      return _objectSpread(_objectSpread({}, Styles.container), style);
    }, [style]);
    return build(SemanticUI)(_objectSpread(_objectSpread({}, restProps), {}, {
      className: mixClass(className, containerClass),
      style: thisStyle,
      id
    }), [build(SemanticUI)({
      key: "doc-body",
      className: "doc-body",
      style: Styles.docBody
    }, [thisBody, thisFooter]), thisMenu, thisRight]);
  };
  return DocTemplate;
};
var defaultStyles = {
  container: {
    margin: 0
  },
  docBody: {
    minWidth: 350
  }
};
export default getDocTemplate;